@@ -22,13 +22,13 @@ from .segment import (create_old_segment_pipe,
                       create_native_old_segment_pipe,
                       create_segment_atropos_pipe,
                       create_segment_atropos_seg_pipe,
-                      create_mask_from_seg_pipe)
+                      create_mask_from_seg_pipe,
+                      create_5tt_pipe)
 
 from .correct_bias import (create_masked_correct_bias_pipe,
                            create_correct_bias_pipe)
 
-from .register import (create_register_NMT_pipe, create_native_iter_reg_pipe,
-                       create_reg_seg_pipe)
+from .register import (create_register_NMT_pipe, create_reg_seg_pipe)
 
 from .extract_brain import (create_extract_pipe,
                             create_extract_T1_pipe)
@@ -121,15 +121,33 @@ def create_full_spm_subpipes(
     outputnode = pe.Node(
         niu.IdentityInterface(fields=['brain_mask', 'segmented_brain_mask']),
         name='outputnode')
-
     # preprocessing
-    if 'short_preparation_pipe' in params.keys():
+    if 'long_single_preparation_pipe' in params.keys():
+        data_preparation_pipe = create_long_single_preparation_pipe(
+            params=parse_key(params, "long_single_preparation_pipe"))
 
+    elif 'long_multi_preparation_pipe' in params.keys():
+        data_preparation_pipe = create_long_multi_preparation_pipe(
+            params=parse_key(params, "long_multi_preparation_pipe"))
+
+    elif 'short_preparation_pipe' in params.keys():
         data_preparation_pipe = create_short_preparation_pipe(
             params=parse_key(params, "short_preparation_pipe"))
+
     else:
-        print("Error, short_preparation_pipe was not \
-            found in params, skipping")
+        print("Error, short_preparation_pipe, long_single_preparation_pipe or\
+            long_multi_preparation_pipe was not found in params, skipping")
+
+        test_node = pe.Node(niu.Function(input_names=['list_T1', 'list_T2'],
+                                         output_names=[''],
+                                         function=list_input_files),
+                            name="test_node")
+
+        seg_pipe.connect(inputnode, 'list_T1',
+                         test_node, 'list_T1')
+        seg_pipe.connect(inputnode, 'list_T2',
+                         test_node, 'list_T2')
+
         return seg_pipe
 
     seg_pipe.connect(inputnode, 'list_T1',
@@ -151,11 +169,12 @@ def create_full_spm_subpipes(
     seg_pipe.connect(inputnode, ('indiv_params', parse_key, "debias"),
                      debias, 'indiv_params')
 
-    if 'crop_T1' in parse_key(params, "short_preparation_pipe"):
-        debias.inputs.bet = 1
-    else:
+    if 'bet_crop' in parse_key(params, "short_preparation_pipe"):
+
         seg_pipe.connect(data_preparation_pipe, 'bet_crop.mask_file',
                          debias, 'b')
+    else:
+        debias.inputs.bet = 1
 
     if pad:
 
@@ -183,82 +202,59 @@ def create_full_spm_subpipes(
         seg_pipe.connect(debias, 'debiased_mask_file',
                          outputnode, "brain_mask")
 
-    if space == "native":
+    # Iterative registration to the INIA19 template
+    reg = NodeParams(IterREGBET(),
+                     params=parse_key(params, "reg"),
+                     name='reg')
 
-        native_iter_reg_pipe = create_native_iter_reg_pipe(
-            params_template,
-            params=parse_key(params, "native_iter_reg_pipe"))
+    reg.inputs.refb_file = params_template["template_brain"]
 
-        seg_pipe.connect(debias, 't1_debiased_brain_file',
-                         native_iter_reg_pipe,
-                         'inputnode.t1_debiased_brain_file')
+    seg_pipe.connect(debias, 't1_debiased_file',
+                     reg, 'inw_file')
 
-        seg_pipe.connect(debias, 't1_debiased_file',
-                         native_iter_reg_pipe, 'inputnode.t1_debiased_file')
+    seg_pipe.connect(debias, 't1_debiased_brain_file',
+                     reg, 'inb_file')
+
+    seg_pipe.connect(inputnode, ('indiv_params', parse_key, "reg"),
+                     reg, 'indiv_params')
+
+    # Compute brain mask using old_segment of SPM and postprocessing on
+    # tissues' masks
+    if "old_segment_pipe" not in params.keys():
+        print("No segmentation, skipping")
+        return seg_pipe
+
+    if space == "template":
+
+        old_segment_pipe = create_old_segment_pipe(
+            params_template, params=parse_key(params, "old_segment_pipe"))
+
+        seg_pipe.connect(reg, 'warp_file',
+                         old_segment_pipe, 'inputnode.T1')
 
         seg_pipe.connect(inputnode, 'indiv_params',
-                         native_iter_reg_pipe, 'inputnode.indiv_params')
+                         old_segment_pipe, 'inputnode.indiv_params')
 
-        # Compute brain mask using old_segment of SPM and postprocessing on
-        # tissues' masks
-        if "old_segment_pipe" in params.keys():
+    elif space == "native":
 
-            old_segment_pipe = create_native_old_segment_pipe(
-                params_template,
-                params=parse_key(params, "old_segment_pipe"))
+        old_segment_pipe = create_native_old_segment_pipe(
+            params_template, params=parse_key(params, "old_segment_pipe"))
 
-            seg_pipe.connect(debias, 't1_debiased_file',
-                             old_segment_pipe, 'inputnode.T1')
+        seg_pipe.connect(reg, 'warp_file',
+                         old_segment_pipe, 'inputnode.T1')
 
-            seg_pipe.connect(native_iter_reg_pipe,
-                             'register_csf_to_nat.out_file',
-                             old_segment_pipe, 'inputnode.native_csf')
+        seg_pipe.connect(reg, 'inv_transfo_file',
+                         old_segment_pipe, 'inputnode.inv_transfo_file')
 
-            seg_pipe.connect(native_iter_reg_pipe,
-                             'register_wm_to_nat.out_file',
-                             old_segment_pipe, 'inputnode.native_wm')
-
-            seg_pipe.connect(native_iter_reg_pipe,
-                             'register_gm_to_nat.out_file',
-                             old_segment_pipe, 'inputnode.native_gm')
-
-            seg_pipe.connect(inputnode, 'indiv_params',
-                             old_segment_pipe, 'inputnode.indiv_params')
-
-        else:
-            return seg_pipe
-
-    elif space == 'template':
-
-        # Iterative registration to the INIA19 template
-        reg = NodeParams(IterREGBET(),
-                         params=parse_key(params, "reg"),
-                         name='reg')
-
-        reg.inputs.refb_file = params_template["template_brain"]
-
-        seg_pipe.connect(debias, 't1_debiased_file', reg, 'inw_file')
         seg_pipe.connect(debias, 't1_debiased_brain_file',
-                         reg, 'inb_file')
+                         old_segment_pipe, 'inputnode.native_T1')
 
-        seg_pipe.connect(inputnode, ('indiv_params', parse_key, "reg"),
-                         reg, 'indiv_params')
+        seg_pipe.connect(inputnode, 'indiv_params',
+                         old_segment_pipe, 'inputnode.indiv_params')
+    else:
 
-        # Compute brain mask using old_segment of SPM and postprocessing on
-        # tissues' masks
-        if "old_segment_pipe" in params.keys():
-
-            old_segment_pipe = create_old_segment_pipe(
-                params_template, params=parse_key(params, "old_segment_pipe"))
-
-            seg_pipe.connect(reg, 'warp_file',
-                             old_segment_pipe, 'inputnode.T1')
-
-            seg_pipe.connect(inputnode, 'indiv_params',
-                             old_segment_pipe, 'inputnode.indiv_params')
-
-        else:
-            return seg_pipe
+        print("Error, space={}".format(space))
+        return seg_pipe
 
     if "mask_from_seg_pipe" in params.keys():
 
@@ -677,9 +673,20 @@ def create_brain_extraction_pipe(params_template, params={},
                                       "smooth_mask.out_file",
                                       outputnode, "brain_mask")
 
-    elif "N4debias" in params.keys():
+    else:
+        if "N4debias" not in params.keys():
+            params["N4debias"] = {
+                "dimension": 3,
+                "bspline_fitting_distance": 200,
+                "n_iterations": [50, 50, 40, 30],
+                "convergence_threshold": 0.00000001,
+                "shrink_factor": 2,
+                "args": "-r 0 --verbose 1"}
 
-        print("Found N4debias")
+            print("Using default bet_crop N4debias: {}".format(
+                params["N4debias"]))
+        else:
+            print("Found N4debias in params.json")
 
         # N4 intensity normalization over T1
         N4debias_T1 = NodeParams(ants.N4BiasFieldCorrection(),
@@ -824,23 +831,40 @@ def create_brain_segment_from_mask_pipe(
         segment_atropos_pipe = create_segment_atropos_pipe(
             params=parse_key(params, "segment_atropos_pipe"))
 
-
         if "use_priors" in params["segment_atropos_pipe"].keys():
 
             brain_segment_pipe.connect(
-                register_NMT_pipe, 'align_seg_csf.out_file', segment_atropos_pipe,
-                "inputnode.csf_prior_file")
+                register_NMT_pipe, 'align_seg_csf.out_file',
+                segment_atropos_pipe, "inputnode.csf_prior_file")
             brain_segment_pipe.connect(
-                register_NMT_pipe, 'align_seg_gm.out_file', segment_atropos_pipe,
-                "inputnode.gm_prior_file")
+                register_NMT_pipe, 'align_seg_gm.out_file',
+                segment_atropos_pipe, "inputnode.gm_prior_file")
             brain_segment_pipe.connect(
-                register_NMT_pipe, 'align_seg_wm.out_file', segment_atropos_pipe,
-                "inputnode.wm_prior_file")
+                register_NMT_pipe, 'align_seg_wm.out_file',
+                segment_atropos_pipe, "inputnode.wm_prior_file")
 
         brain_segment_pipe.connect(
             register_NMT_pipe, 'norm_intensity.output_image',
             segment_atropos_pipe, "inputnode.brain_file")
 
+    if "export_5tt_pipe" in params.keys():
+
+        export_5tt_pipe = create_5tt_pipe(
+            params=parse_key(params, "export_5tt_pipe"))
+
+        brain_segment_pipe.connect(segment_atropos_pipe,
+                                   'outputnode.threshold_gm',
+                                   export_5tt_pipe, 'inputnode.gm_file')
+
+        brain_segment_pipe.connect(segment_atropos_pipe,
+                                   'outputnode.threshold_wm',
+                                   export_5tt_pipe, 'inputnode.wm_file')
+
+        brain_segment_pipe.connect(segment_atropos_pipe,
+                                   'outputnode.threshold_csf',
+                                   export_5tt_pipe, 'inputnode.csf_file')
+
+    # output
     outputnode = pe.Node(
         niu.IdentityInterface(
             fields=["segmented_file", "threshold_gm", "threshold_wm",
@@ -1255,7 +1279,6 @@ def create_brain_segment_from_mask_T1_pipe(
             fields=['preproc_T1', 'brain_mask', 'indiv_params']),
         name='inputnode')
 
-
     # creating outputnode
     outputnode = pe.Node(
         niu.IdentityInterface(
@@ -1300,12 +1323,14 @@ def create_brain_segment_from_mask_T1_pipe(
     if "use_priors" in params["segment_atropos_pipe"].keys():
 
         brain_segment_pipe.connect(register_NMT_pipe, 'align_seg_csf.out_file',
-                                segment_atropos_pipe,
-                                "inputnode.csf_prior_file")
+                                   segment_atropos_pipe,
+                                   "inputnode.csf_prior_file")
         brain_segment_pipe.connect(register_NMT_pipe, 'align_seg_gm.out_file',
-                                segment_atropos_pipe, "inputnode.gm_prior_file")
+                                   segment_atropos_pipe,
+                                   "inputnode.gm_prior_file")
         brain_segment_pipe.connect(register_NMT_pipe, 'align_seg_wm.out_file',
-                                segment_atropos_pipe, "inputnode.wm_prior_file")
+                                   segment_atropos_pipe,
+                                   "inputnode.wm_prior_file")
 
     if space == 'native':
 
@@ -1448,12 +1473,12 @@ def create_full_T1_ants_subpipes(params_template, params={},
         params=parse_key(params, "brain_segment_pipe"), space=space)
 
     seg_pipe.connect(data_preparation_pipe, 'outputnode.preproc_T1',
-                        brain_segment_pipe, 'inputnode.preproc_T1')
+                     brain_segment_pipe, 'inputnode.preproc_T1')
     seg_pipe.connect(brain_extraction_pipe,
-                        "extract_T1_pipe.smooth_mask.out_file",
-                        brain_segment_pipe, "inputnode.brain_mask")
+                     "extract_T1_pipe.smooth_mask.out_file",
+                     brain_segment_pipe, "inputnode.brain_mask")
     seg_pipe.connect(inputnode, 'indiv_params',
-                        brain_segment_pipe, 'inputnode.indiv_params')
+                     brain_segment_pipe, 'inputnode.indiv_params')
 
     if pad and space == "native":
 
@@ -1481,7 +1506,6 @@ def create_full_T1_ants_subpipes(params_template, params={},
         seg_pipe.connect(brain_segment_pipe, 'outputnode.segmented_file',
                          outputnode, 'segmented_brain_mask')
 
-
     if "nii_to_mesh_fs_pipe" in params.keys():
         nii_to_mesh_fs_pipe = create_nii_to_mesh_fs_pipe(
             params=parse_key(params, "nii_to_mesh_fs_pipe"))
